## $Id: browser.py 14964 2018-03-08 20:42:40Z henrik $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import httplib
import hashlib
import grok
from kofacustom.nigeria.interswitch.browser import (
    InterswitchPaymentRequestWebservicePageApplicant,
    InterswitchPaymentRequestWebservicePageStudent,
    InterswitchPaymentVerifyWebservicePageApplicant,
    InterswitchPaymentVerifyWebservicePageStudent,
    InterswitchPageStudent, InterswitchPageApplicant,
    )
from kofacustom.dspg.students.interfaces import ICustomStudentOnlinePayment
from kofacustom.dspg.applicants.interfaces import ICustomApplicantOnlinePayment
from kofacustom.dspg.interfaces import MessageFactory as _

PRODUCT_ID = '7269' # must be provided by Interswitch
SITE_NAME = 'dspg.waeup.org'
PROVIDER_ACCT = '2028964403'
PROVIDER_BANK_ID = '8'
PROVIDER_ITEM_NAME = 'BT Education'
INSTITUTION_NAME = 'Delta State Polytechnic Ogwashi-Uku'
CURRENCY = '566'
GATEWAY_AMT = 250.0

POST_ACTION = 'https://webpay.interswitchng.com/paydirect/pay'
#POST_ACTION = 'https://sandbox.interswitchng.com/webpay/pay'
HOST = 'webpay.interswitchng.com'
#HOST = 'sandbox.interswitchng.com'
URL = '/paydirect/api/v1/gettransaction.json'
#URL = '/webpay/api/v1/gettransaction.json'

httplib.HTTPSConnection.debuglevel = 0
HTTPS = True

SPECIAL_PAYMENT_PARAMS = {

    'carryover1': ('101', 500.0, '1012808851', '117'),
    'carryover5': ('101', 500.0, '1012808851', '117'),
    'carryover6': ('101', 500.0, '1012808851', '117'),
    'carryover7': ('101', 500.0, '1012808851', '117'),
    'carryover8': ('101', 500.0, '1012808851', '117'),
    'carryover9': ('101', 500.0, '1012808851', '117'),
    'carryover10': ('101', 500.0, '1012808851', '117'),
    'carryover11': ('101', 500.0, '1012808851', '117'),
    'carryover12': ('101', 500.0, '1012808851', '117'),
    'balance': ('101', 500.0, '1012808851', '117'),

    'certificate': ('111', 500.0, '1012808851', '117'),
    'state_result': ('112', 500.0, '1012808851', '117'),
    'transcript_local': ('113', 500.0, '1012808851', '117'),
    'transcript_foreign': ('114', 500.0, '1012808851', '117'),
    'ver_result': ('115', 500.0, '1012808851', '117'),
    'change_course': ('116', 500.0, '1012808851', '117'),
    'change_inst': ('117', 500.0, '1012808851', '117'),
    'jamb_reject': ('118', 500.0, '1012808851', '117'),
    'cert_of_cert': ('119', 500.0, '1012808851', '117'),
    'ref_let': ('120', 500.0, '1012808851', '117'),
    'proc_cert': ('121', 500.0, '1012808851', '117'),
    'loss_idcard': ('122', 9.0, '1012808851', '117'),
    'loss_examcard': ('123', 4.5, '1012808851', '117'),
    'loss_result': ('124', 500.0, '1012808851', '117'),
    'loss_receipt': ('125', 500.0, '1012808851', '117'),
    'loss_clearance': ('126', 500.0, '1012808851', '117'),
    'conv_brochure': ('127', 500.0, '1012808851', '117'),
    'hnd_certificate': ('128', 500.0, '1012808851', '117'),
    'hnd_state_result': ('129', 500.0, '1012808851', '117'),
    'hnd_transcript_local': ('130', 500.0, '1012808851', '117'),
    'hnd_transcript_foreign': ('131', 500.0, '1012808851', '117'),
    'staff_loss_idcard': ('132', 9.0, '1012808851', '117'),
    'hnd_conv_brochure': ('133', 500.0, '1012808851', '117'),
    'pgd_state_result': ('134', 500.0, '1012808851', '117'),
    'pgd_conv_brochure': ('135', 500.0, '1012808851', '117'),
    'pgd_certificate': ('136', 500.0, '1012808851', '117'),
    'log_book': ('137', 4.5, '1012808851', '117'),
    'pgd_transcript_foreign': ('138', 500.0, '1012808851', '117'),
    'pgd_transcript_local': ('139', 500.0, '1012808851', '117'),
    'jamb_regularization': ('140', 500.0, '1012808851', '117'),
    'utme_registration': ('142', 9.0, '1012808851', '117'),
    'utme_cbt': ('143', 9.0, '1012808851', '117'),
    'nysc_id_card': ('144', 9.0, '1012808851', '117'),
    'ijmb_result': ('147', 500.0, '1012808851', '117'),
    }


# Not yet readily configured. Bank account numbers are
# contradictory. pay_item_ids are not yet assigned.

class CustomInterswitchPageStudent(InterswitchPageStudent):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomStudentOnlinePayment)
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    product_id = PRODUCT_ID
    mac = 'D43ED93B7A307B152C7111D04E1F384428057CD1E75428D3A4C5631C28ACB9C8C822A88CA76DEDC5F9A520CF4BFDB2824FF8B207AAFC667DF3CBB13685B66627'
    pay_item_id = '000'

    def update(self):
        error = self.init_update()
        if error:
            self.flash(error, type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        student = self.student
        xmldict = self.xmldict
        # Provider data
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        # Institution data
        xmldict['institution_acct'] = '00000000'
        xmldict['institution_bank_id'] = '00'
        xmldict['institution_amt'] = '0.0'
        provider_amt = 0.0
        tech_fee = 0.0
        if self.context.p_category.startswith('schoolfee'):
            self.pay_item_id = '102'
            provider_amt = 2800.0
            tech_fee = 1200.0
            if student.faccode == 'SPAT':
                xmldict['institution_acct'] = '1012808851'
                xmldict['institution_bank_id'] = '117'
            elif student.current_mode in ('nd_ft', 'hnd_ft'):
                xmldict['institution_acct'] = '0010761873'
                xmldict['institution_bank_id'] = '11'
        elif self.context.p_category == 'clearance':
            self.pay_item_id = '103'
            provider_amt = 500.0
            if student.faccode == 'SPAT':
                xmldict['institution_acct'] = '1012808851'
                xmldict['institution_bank_id'] = '117'
            elif student.current_mode == 'nd_ft':
                xmldict['institution_acct'] = '2004402644'
                xmldict['institution_bank_id'] = '8'
        elif self.context.p_category == 'gown':
            self.pay_item_id = '102' # We use school fee item id.
            xmldict['institution_acct'] = '2006612782'
            xmldict['institution_bank_id'] = '8'
        elif self.context.p_category in SPECIAL_PAYMENT_PARAMS.keys():
            self.pay_item_id = SPECIAL_PAYMENT_PARAMS[self.context.p_category][0]
            provider_amt = SPECIAL_PAYMENT_PARAMS[self.context.p_category][1]
            xmldict['institution_acct'] = SPECIAL_PAYMENT_PARAMS[self.context.p_category][2]
            xmldict['institution_bank_id'] = SPECIAL_PAYMENT_PARAMS[self.context.p_category][3]
        xmldict['provider_amt'] = 100 * provider_amt
        xmldict['tech_fee'] = 100 * tech_fee
        xmldict['institution_item_name'] = self.context.category
        xmldict['institution_name'] = INSTITUTION_NAME
        xmldict['institution_amt'] = institution_amt = 100 * (
            self.context.amount_auth - provider_amt - tech_fee - GATEWAY_AMT)

        if provider_amt == 0:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        elif tech_fee == 0:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        else:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
<item_detail item_id="3" item_name="Technology Fee" item_amt="%(tech_fee)d" bank_id="307" acct_num="5100189030" />
</item_details>
</payment_item_detail>""" % xmldict

        # isouaba: All certificates in SPAT are part time thus any payments
        # coming from those certificates (Application, Acceptance, school fees
        # etc) 40% should be split to keystone account.
        if student.faccode == 'SPAT':
            xmldict['institution_amt_2'] = 0.4 * institution_amt
            xmldict['institution_amt_1'] = 0.6 * institution_amt
            if provider_amt == 0:
                xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s 1" item_amt="%(institution_amt_1)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(institution_item_name)s 2" item_amt="%(institution_amt_2)d" bank_id="123" acct_num="1002883141" />
</item_details>
</payment_item_detail>""" % xmldict
            elif tech_fee == 0:
                xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s 1" item_amt="%(institution_amt_1)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(institution_item_name)s 2" item_amt="%(institution_amt_2)d" bank_id="123" acct_num="1002883141" />
<item_detail item_id="3" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
            else:
                xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s 1" item_amt="%(institution_amt_1)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(institution_item_name)s 2" item_amt="%(institution_amt_2)d" bank_id="123" acct_num="1002883141" />
<item_detail item_id="3" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
<item_detail item_id="4" item_name="Technology Fee" item_amt="%(tech_fee)d" bank_id="307" acct_num="5100189030" />
</item_details>
</payment_item_detail>""" % xmldict
        # dep_sug payments are treated completely separately. The following lines
        # override the configuration above.
        if self.context.p_category == 'dep_sug':
            self.pay_item_id = '102' # We use school fee item id
            # amount_auth is being ignored. However, the sum of all split payments
            # must coincide with the amount_auth configured in students/utils.py
            xmldict['amt1'] = 100 * 1000
            xmldict['amt2'] = 100 * 500
            xmldict['amt3'] = 100 * 900
            xmldict['amt4'] = 100 * 500

            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="SUG" item_amt="%(amt1)d" bank_id="11" acct_num="0038079930" />
<item_detail item_id="2" item_name="Students Welfare" item_amt="%(amt2)d" bank_id="11" acct_num="0037892949" />
<item_detail item_id="3" item_name="Anti-Cult Book" item_amt="%(amt3)d" bank_id="11" acct_num="0037892949" />
<item_detail item_id="4" item_name="NADESSTU" item_amt="%(amt4)d" bank_id="11" acct_num="0036375968" />
</item_details>
</payment_item_detail>""" % xmldict

#            if student.faccode == 'SPAT':
#                xmltext = """<payment_item_detail>
#<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
#<item_detail item_id="1" item_name="Students Welfare" item_amt="%(amt2)d" bank_id="11" acct_num="0037892949" />
#<item_detail item_id="2" item_name="Anti-Cult Book" item_amt="%(amt3)d" bank_id="11" acct_num="0037892949" />
#</item_details>
#</payment_item_detail>""" % xmldict

        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.context.provider_amt = provider_amt
        self.context.gateway_amt = GATEWAY_AMT
        hashargs = (
            self.context.p_id +
            PRODUCT_ID +
            self.pay_item_id +
            str(int(self.amount_auth)) +
            self.site_redirect_url +
            self.mac)
        self.hashvalue = hashlib.sha512(hashargs).hexdigest()
        return

class CustomInterswitchPageApplicant(InterswitchPageApplicant):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomApplicantOnlinePayment)
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    pay_item_id = '101' # must be provided by Interswitch
    product_id = PRODUCT_ID
    mac = 'D43ED93B7A307B152C7111D04E1F384428057CD1E75428D3A4C5631C28ACB9C8C822A88CA76DEDC5F9A520CF4BFDB2824FF8B207AAFC667DF3CBB13685B66627'

    def update(self):
        error = self.init_update()
        if error:
            self.flash(error, type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        xmldict = {}
        provider_amt = 500.0
        gateway_amt = GATEWAY_AMT
        xmldict['institution_acct'] = '00000000'
        xmldict['institution_bank_id'] = '00'
        if self.applicant.__parent__.application_category in ('ndft', 'hndft'):
            xmldict['institution_acct'] = '0010761873'
            xmldict['institution_bank_id'] = '11'
        elif self.applicant.__parent__.application_category in (
            'ndpt', 'hndpt', 'ndwe', 'hndwe'):
            xmldict['institution_acct'] = '1015220292'
            xmldict['institution_bank_id'] = '117'
        elif self.applicant.applicant_id.startswith('special'):
            if self.context.p_category in SPECIAL_PAYMENT_PARAMS.keys():
                self.pay_item_id = SPECIAL_PAYMENT_PARAMS[self.context.p_category][0]
                provider_amt = SPECIAL_PAYMENT_PARAMS[self.context.p_category][1]
                xmldict['institution_acct'] = SPECIAL_PAYMENT_PARAMS[self.context.p_category][2]
                xmldict['institution_bank_id'] = SPECIAL_PAYMENT_PARAMS[self.context.p_category][3]
        if self.pay_item_id in ('122', '123', '132', '137', '141',
                                '142', '143', '144', '146', '147',):
                gateway_amt = round(0.015 * self.context.amount_auth, 2)
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_amt'] = 100 * provider_amt
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        xmldict['institution_amt'] = institution_amt = 100 * (
            self.context.amount_auth - provider_amt - gateway_amt)
        xmldict['institution_item_name'] = self.context.category
        xmldict['institution_name'] = INSTITUTION_NAME
        xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict

        # isouaba: All certificates in SPAT are part time thus any payments
        # coming from those certificates (Application, Acceptance, school fees
        # etc) 40% should be split to keystone account.
        if self.applicant.__parent__.application_category in (
            'ndpt', 'hndpt', 'ndwe', 'hndwe'):
            xmldict['institution_amt_2'] = 0.4 * institution_amt
            xmldict['institution_amt_1'] = 0.6 * institution_amt
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s">
<item_detail item_id="1" item_name="%(institution_item_name)s 1" item_amt="%(institution_amt_1)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(institution_item_name)s 2" item_amt="%(institution_amt_2)d" bank_id="123" acct_num="1002883141" />
<item_detail item_id="3" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict

        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.context.provider_amt = provider_amt
        self.context.gateway_amt = GATEWAY_AMT

        hashargs = (
            self.context.p_id +
            PRODUCT_ID +
            self.pay_item_id +
            str(int(self.amount_auth)) +
            self.site_redirect_url +
            self.mac)
        self.hashvalue = hashlib.sha512(hashargs).hexdigest()
        return

class CustomInterswitchPaymentRequestWebservicePageStudent(
    InterswitchPaymentRequestWebservicePageStudent):
    """Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomStudentOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    mac = 'D43ED93B7A307B152C7111D04E1F384428057CD1E75428D3A4C5631C28ACB9C8C822A88CA76DEDC5F9A520CF4BFDB2824FF8B207AAFC667DF3CBB13685B66627'

class CustomInterswitchPaymentVerifyWebservicePageStudent(
    InterswitchPaymentVerifyWebservicePageStudent):
    """Payment verify view for the CollegePAY gateway
    """
    grok.context(ICustomStudentOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    mac = 'D43ED93B7A307B152C7111D04E1F384428057CD1E75428D3A4C5631C28ACB9C8C822A88CA76DEDC5F9A520CF4BFDB2824FF8B207AAFC667DF3CBB13685B66627'

class CustomInterswitchPaymentRequestWebservicePageApplicant(
    InterswitchPaymentRequestWebservicePageApplicant):
    """Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomApplicantOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    mac = 'D43ED93B7A307B152C7111D04E1F384428057CD1E75428D3A4C5631C28ACB9C8C822A88CA76DEDC5F9A520CF4BFDB2824FF8B207AAFC667DF3CBB13685B66627'

class CustomInterswitchPaymentVerifyWebservicePageApplicant(
    InterswitchPaymentVerifyWebservicePageApplicant):
    """Payment verify view for the CollegePAY gateway
    """
    grok.context(ICustomApplicantOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    mac = 'D43ED93B7A307B152C7111D04E1F384428057CD1E75428D3A4C5631C28ACB9C8C822A88CA76DEDC5F9A520CF4BFDB2824FF8B207AAFC667DF3CBB13685B66627'